# Autogenerated by Astropy-affiliated package so_pysm_models's setup.py on 2021-02-12 19:20:29 UTC
from __future__ import unicode_literals
import datetime

version = "2.4.0"
githash = "562e71d2f6405ef50134c696dc8a7a9ee1eb5d20"


major = 2
minor = 4
bugfix = 0

version_info = (major, minor, bugfix)

release = True
timestamp = datetime.datetime(2021, 2, 12, 19, 20, 29)
debug = False

astropy_helpers_version = "3.0.2"
