# Autogenerated by Astropy-affiliated package astropy_helpers's setup.py on 2019-01-25 03:21:16 UTC
from __future__ import unicode_literals
import datetime

version = "3.0.2"
githash = "ca630405b54d40695b7bbd2824cb0f142b5811b6"


major = 3
minor = 0
bugfix = 2

release = True
timestamp = datetime.datetime(2019, 1, 25, 3, 21, 16)
debug = False

astropy_helpers_version = ""

try:
    from ._compiler import compiler
except ImportError:
    compiler = "unknown"

try:
    from .cython_version import cython_version
except ImportError:
    cython_version = "unknown"
